/*
 * Copyright (C) 2012 Google Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer
 *    in the documentation and/or other materials provided with the
 *    distribution.
 * 3. Neither the name of Google Inc. nor the names of its contributors
 *    may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef RTCSessionDescription_h
#define RTCSessionDescription_h

#if ENABLE(MEDIA_STREAM)

#include "ExceptionCode.h"
#include "ScriptWrappable.h"
#include <wtf/RefCounted.h>
#include <wtf/RefPtr.h>
#include <wtf/text/WTFString.h>

namespace WebCore {

class Dictionary;

class RTCSessionDescription : public RefCounted<RTCSessionDescription>, public ScriptWrappable {
public:
    static RefPtr<RTCSessionDescription> create(const Dictionary&, ExceptionCode&);
    static Ref<RTCSessionDescription> create(const RTCSessionDescription*);
    static Ref<RTCSessionDescription> create(const String& type, const String& sdp);
    virtual ~RTCSessionDescription() { }

    const String& type() const { return m_type; }
    void setType(const String&, ExceptionCode&);

    const String& sdp() const { return m_sdp; }
    void setSdp(const String& sdp) { m_sdp = sdp; }

private:
    explicit RTCSessionDescription(const String& type, const String& sdp);

    String m_type;
    String m_sdp;
};

} // namespace WebCore

#endif // ENABLE(MEDIA_STREAM)

#endif // RTCSessionDescription_h
