/*
 * Copyright (C) 2013-2015 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

WebInspector.ImageResourceContentView = class ImageResourceContentView extends WebInspector.ResourceContentView
{
    constructor(resource)
    {
        super(resource, "image");

        this._imageElement = null;
    }

    // Public

    get imageElement()
    {
        return this._imageElement;
    }

    contentAvailable(content, base64Encoded)
    {
        this.element.removeChildren();

        var objectURL = this.resource.createObjectURL();
        this._imageElement = document.createElement("img");
        this._imageElement.dataset.url = this.resource.url;

        var imageElement = this._imageElement;
        var xhr = new XMLHttpRequest();
        xhr.open("GET", objectURL, true);
        xhr.responseType = "blob";
        xhr.onload = function() {
            if (this.status == 200) {
                var reader = new FileReader();
                reader.onload = function() {
                    imageElement.src = reader.result;

                    URL.revokeObjectURL(objectURL);
                }

                reader.readAsDataURL(this.response);
            } else {
                imageElement.addEventListener("load", function() { URL.revokeObjectURL(objectURL) });
                imageElement.src = objectURL;
            }
        };
        xhr.send();

        this.element.appendChild(this._imageElement);
    }
};
